<?php
/**
 * @file nodereference_embed.inc
 * API calls and plugin logic is placed into this file.
 */


/**
 * We search the text for matching tokens and replace it with their themed HTML representations.
 *
 * @param $text
 *   The text fragment to be searched for an embed token
 */
function nodereference_embed_process_tokens(&$text) {

  $plugin_types = array_keys(nodereference_embed_get_plugins());

  if (!empty($plugin_types)) {
    foreach ($plugin_types as $plugin_type) {
      // search for all nodereference embed tokens in provided text
      if (preg_match_all("/\[\s*(node reference embed:$plugin_type|$plugin_type)\s+id=(\d+)\s*[\||\s]?\s*([^\]]*)\s*\]/", $text, $matches)) {
        for ($i = 0; $i < count($matches[0]); $i++) {
          if (!empty($matches[1][$i]) && !empty($matches[2][$i])) {
            $options = array();

            // parse token arguments into options array
            $matches[3][$i] = htmlspecialchars_decode($matches[3][$i]); // decode ampersands & if used (WYSIWIG)
            if (preg_match_all('/\s*&?([^=\s&]*)\s*=\s*("[^="&]*"|[^="\s&]*)\s*/', $matches[3][$i], $sub_matches)) {
              foreach ($sub_matches[2] as &$sub_match) { // strip quotes
                $sub_match = str_replace('"', '', $sub_match);
              }
              $options = array_combine($sub_matches[1], $sub_matches[2]);
            }

            $nid = $matches[2][$i]; // node id
            $text = str_replace($matches[0][$i], '[' . $matches[1][$i] .']', $text);
            // replace token with actual value
            $text = token_replace($text, 'node reference embed', $nid, TOKEN_PREFIX, TOKEN_SUFFIX, $options);
          }
        }
      }
    }
  }
}

/**
 * Process the Embed plugin, i. e. load the proper plugin class and theme the token.
 *
 * @param $type
 *   plugin type, e. g. "node"
 * @param $id
 *   identified, e. g. node id
 * @param $options
 *   optional arguments array for theming
 * @return
 *   HTML string output
 */
function nodereference_embed_process_plugin($type, $id, $options = array()) {
  // Load all available plugins
  $plugins = nodereference_embed_get_plugins($type);

  // Object factory: get the default display of plugin, first user input, then configuration and as a fallback the default value
  $display = $options['display'] ? $options['display'] : variable_get("nodereference_embed_plugin_{$type}_default_display", 'default');

  $plugin = $plugins[$display]; // get plugin with proper display
  if (is_object($plugin) && is_subclass_of($plugin, 'nodereference_embed_plugin')) {
    $plugin->load($id); // Load data
    return $plugin->render($options); // Theme output
  }
}

/**
 * Load plugin objects.
 *
 * @param string $type
 *   limit by plugin type, e. g. "node"
 * @return
 *   array of plugin objects
 */
function nodereference_embed_get_plugins($type = 'all') {

  static $plugins = array(); // cache plugin info
  if (empty($plugins)) {
    // load available plugins, it's mostly about loading the proper inc files and object initiation.
    $hooks = nodereference_embed_invoke_plugin_hooks();
    if (!empty($hooks)) {
      // global interface, always resides in parent module
      module_load_include('inc', 'nodereference_embed', 'plugins/nodereference_embed_plugin');
      foreach ($hooks as $plugin_type => $displays) {
        // load inc for super class of plugin type, usually in parent module
        module_load_include('inc', 'nodereference_embed', "plugins/nodereference_embed_plugin_$plugin_type");
        foreach ($displays as $key => $display) {
          $handler_inc = $display['parent']; // name of handler class
          $parent_inc = $display['handler'];
          $type_inc = "nodereference_embed_plugin_$plugin_type"; // if super class resides in other module
          if ($display['path']) {
            $handler_inc = $display['path'] . '/' . $handler_inc;
            $parent_inc = $display['path'] . '/' . $parent_inc;
            $type_inc = $display['path'] . '/' . $type_inc;
          }
          // we examine all hooks and load the inc files in all possible locations
          foreach (module_implements('nodereference_embed_plugins') as $module) {
            module_load_include('inc', $module, $handler_inc);
            module_load_include('inc', $module, $parent_inc);
            module_load_include('inc', $module, $type_inc);
          }
          // store and initiate plugin object if plugin class exists
          if (class_exists($display['handler'])) {
            if ($display['title']) { // title is important for settings page
              $plugins[$plugin_type][$key] = new $display['handler']($display['title']);
            }
            else {
              $plugins[$plugin_type][$key] = new $display['handler']($key);
            }
          }
        }
      }
    }
  }
  return $type == 'all' ? $plugins : $plugins[$type];
}

/**
 * Collect all modules implementing the plugin info hook.
 * @return
 *   array of hook info
 */
function nodereference_embed_invoke_plugin_hooks() {
  static $hooks = array();
  if (empty($hooks)) {
    $hooks = module_invoke_all('nodereference_embed_plugins');
  }
  return $hooks;
}

/**
 * Administration settings form.
 *
 * @return array
 *   form array
 */
function nodereference_embed_settings() {
  $form = array();
  // load all settings form of all plugins into a single form
  $plugins = nodereference_embed_get_plugins();
  foreach ($plugins as $plugin_type => $displays) {
    foreach ($displays as $display) {
      $form += $display->settings($displays); // add form
    }
  }
  return system_settings_form($form);
}